#include "GestorTemes.h"

#include <string.h>
#include <stdio.h>
#include <dirent.h>
#include <stdlib.h>
#include <ctype.h>
#include <iostream>
#ifdef TARGET_WIN32
#   include <windows.h>
#endif
#include "SDL_image.h"

using namespace std;

#ifndef TARGET_WIN32
extern int alphasort();
#endif


GestorTemes::GestorTemes()
{
    mNumTemes=0;

#ifndef TARGET_WIN32
    struct dirent **llista;
    int n=scandir("data/themes", &llista, NULL, alphasort);

    for (Uint8 i=0; i<n; i++)
    {
        if ((strcmp(llista[i]->d_name, ".")!=0) && (strcmp(llista[i]->d_name, "..")!=0))
        {
            mTemes[mNumTemes]=new char[strlen(llista[i]->d_name)+1];
            strcpy(mTemes[mNumTemes], llista[i]->d_name);
            mNumTemes++;
        }
        free(llista[i]);
    }
#else
    WIN32_FIND_DATA dades;
    HANDLE directoris = FindFirstFile("data/themes/*", &dades);

    if (directoris==INVALID_HANDLE_VALUE)
    {
        cerr << "Error llegint <data/themes>" << endl;
        exit(1);
    }
    else
    {
        if ((strcmp(dades.cFileName, ".")!=0) && (strcmp(dades.cFileName, "..")!=0))
        {
            mTemes[mNumTemes]=new char[strlen(dades.cFileName)+1];
            strcpy(mTemes[mNumTemes], dades.cFileName);
            mNumTemes++;
        }
        while (FindNextFile(directoris, &dades)!=0)
        {
            if ((strcmp(dades.cFileName, ".")!=0) && (strcmp(dades.cFileName, "..")!=0))
            {
                mTemes[mNumTemes]=new char[strlen(dades.cFileName)+1];
                strcpy(mTemes[mNumTemes], dades.cFileName);
                mNumTemes++;
            }
        }
        FindClose(directoris);
    }
#endif
}


GestorTemes::~GestorTemes()
{
    for (Uint8 i=0; i<mNumTemes; i++) delete mTemes[i];
}


void GestorTemes::CarregaTema(Uint8 tema,
                              SDL_Surface** peces,
                              SDL_Surface** pecesRestants,
                              SDL_Surface** fons,
                              SDL_Surface** cursor,
                              Mix_Music** soMenu,
                              Mix_Music** soPuntuacions,
                              Mix_Music** soRecord,
                              Mix_Chunk** soCursor,
                              Mix_Chunk** soMarca,
                              Mix_Chunk** soCanvi,
                              Mix_Chunk** soNoCanvi,
                              Mix_Chunk** soDeuSegons,
                              Mix_Chunk** soNoMesLinies,
                              Mix_Chunk** soFiNivell,
                              Mix_Chunk** soFiPartida,
                              Mix_Chunk** soPausa,
                              SDL_Surface* superficie)
{
    char sDirectori[128], sDirectoriDefecte[128], sPeces[128], sPecesRestants[128], sFons[128], sCursor[128];

    sprintf(sDirectori, "data/themes/%s/", mTemes[tema]);
    sprintf(sDirectoriDefecte, "data/themes/default/");

    sprintf(sPeces, "%s%s", sDirectori, "tiles-board.png");
    SDL_Surface* imgPeces = IMG_Load(sPeces);
    if (imgPeces == NULL)
    {
        sprintf(sPeces, "%s%s", sDirectoriDefecte, "tiles-board.png");
        imgPeces = IMG_Load(sPeces);
        if (imgPeces == NULL)
        {
            cerr << "Error llegint <" << sPeces << ">: " << SDL_GetError() << endl;
            exit(1);
        }
    }
    SDL_SetColorKey(imgPeces, SDL_SRCCOLORKEY, SDL_MapRGB(superficie->format, 1, 0, 0));
    *peces = SDL_DisplayFormat(imgPeces);
    SDL_FreeSurface(imgPeces);

    sprintf(sPecesRestants, "%s%s", sDirectori, "tiles-remaining.png");
    SDL_Surface* imgPecesRestants = IMG_Load(sPecesRestants);
    if (imgPecesRestants == NULL)
    {
        sprintf(sPecesRestants, "%s%s", sDirectoriDefecte, "tiles-remaining.png");
        imgPecesRestants = IMG_Load(sPecesRestants);
        if (imgPecesRestants == NULL)
        {
            cerr << "Error llegint <" << sPecesRestants << ">: " << SDL_GetError() << endl;
            exit(1);
        }
    }
    SDL_SetColorKey(imgPecesRestants, SDL_SRCCOLORKEY, SDL_MapRGB(superficie->format, 1, 0, 0));
    *pecesRestants = SDL_DisplayFormat(imgPecesRestants);
    SDL_FreeSurface(imgPecesRestants);

    sprintf(sFons, "%s%s", sDirectori, "background-menu.png");
    SDL_Surface* imgFons = IMG_Load(sFons);
    if (imgFons == NULL)
    {
        sprintf(sFons, "%s%s", sDirectoriDefecte, "background-menu.png");
        imgFons = IMG_Load(sFons);
        if (imgFons == NULL)
        {
            cerr << "Error llegint <" << sFons << ">: " << SDL_GetError() << endl;
            exit(1);
        }
    }
    SDL_SetColorKey(imgFons, SDL_SRCCOLORKEY, SDL_MapRGB(superficie->format, 1, 0, 0));
    *fons = SDL_DisplayFormat(imgFons);
    SDL_FreeSurface(imgFons);

    sprintf(sCursor, "%s%s", sDirectori, "cursor.png");
    SDL_Surface* imgCursor = IMG_Load(sCursor);
    if (imgCursor == NULL)
    {
        sprintf(sCursor, "%s%s", sDirectoriDefecte, "cursor.png");
        imgCursor = IMG_Load(sCursor);
        if (imgCursor == NULL)
        {
            cerr << "Error llegint <" << sCursor << ">: " << SDL_GetError() << endl;
            exit(1);
        }
    }
    SDL_SetColorKey(imgCursor, SDL_SRCCOLORKEY, SDL_MapRGB(superficie->format, 1, 0, 0));
    *cursor = SDL_DisplayFormat(imgCursor);
    SDL_FreeSurface(imgCursor);

    char sSoMenu[128],  sSoCursor[128], sSoMarca[128];
    char sSoCanvi[128], sSoNoCanvi[128], sSoDeuSegons[128], sSoNoMesLinies[128], sSoFiNivell[128];
    char sSoFiPartida[128], sSoPausa[128];

    sprintf(sSoMenu, "%s%s", sDirectori, "sound-cursor.ogg.wav");
    *soMenu = Mix_LoadMUS(sSoMenu);
    if (*soMenu==NULL)
    {
        sprintf(sSoMenu, "%s%s", sDirectoriDefecte, "sound-cursor.ogg.wav");
        *soMenu = Mix_LoadMUS(sSoMenu);
        if (*soMenu==NULL)
        {
            cerr << "Error llegint <" << sSoMenu << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
          sprintf(sSoCursor, "%s%s", sDirectori, "sound-cursor.ogg.wav");
    *soCursor = Mix_LoadWAV(sSoCursor);
    if (*soCursor==NULL)
    {
        sprintf(sSoCursor, "%s%s", sDirectoriDefecte, "sound-cursor.ogg.wav");
        *soCursor = Mix_LoadWAV(sSoCursor);
        if (*soCursor==NULL)
        {
            cerr << "Error llegint <" << sSoCursor << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
    sprintf(sSoMarca, "%s%s", sDirectori, "sound-select.ogg.wav");
    *soMarca = Mix_LoadWAV(sSoMarca);
    if (soMarca==NULL)
    {
        sprintf(sSoMarca, "%s%s", sDirectoriDefecte, "sound-select.ogg.wav");
        *soMarca = Mix_LoadWAV(sSoMarca);
        if (*soMarca==NULL)
        {
            cerr << "Error llegint <" << sSoMarca << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
    sprintf(sSoCanvi, "%s%s", sDirectori, "sound-match.ogg.wav");
    *soCanvi = Mix_LoadWAV(sSoCanvi);
    if (*soCanvi==NULL)
    {
        sprintf(sSoCanvi, "%s%s", sDirectoriDefecte, "sound-match.ogg.wav");
        *soCanvi = Mix_LoadWAV(sSoCanvi);
        if (*soCanvi==NULL)
        {
            cerr << "Error llegint <" << sSoCanvi << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
    sprintf(sSoNoCanvi, "%s%s", sDirectori, "sound-nomatch.ogg.wav");
    *soNoCanvi = Mix_LoadWAV(sSoNoCanvi);
    if (*soNoCanvi==NULL)
    {
        sprintf(sSoNoCanvi, "%s%s", sDirectoriDefecte, "sound-nomatch.ogg.wav");
        *soNoCanvi = Mix_LoadWAV(sSoNoCanvi);
        if (*soNoCanvi==NULL)
        {
            cerr << "Error llegint <" << sSoNoCanvi << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
    sprintf(sSoFiNivell, "%s%s", sDirectori, "sound-endlevel.ogg.wav");
    *soFiNivell = Mix_LoadWAV(sSoFiNivell);
    if (*soFiNivell==NULL)
    {
        sprintf(sSoFiNivell, "%s%s", sDirectoriDefecte, "sound-endlevel.ogg.wav");
        *soFiNivell = Mix_LoadWAV(sSoFiNivell);
        if (*soFiNivell==NULL)
        {
            cerr << "Error llegint <" << sSoFiNivell << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
    sprintf(sSoDeuSegons, "%s%s", sDirectori, "sound-tenseconds.ogg.wav");
    *soDeuSegons = Mix_LoadWAV(sSoDeuSegons);
    if (*soDeuSegons==NULL)
    {
        sprintf(sSoDeuSegons, "%s%s", sDirectoriDefecte, "sound-tenseconds.ogg.wav");
        *soDeuSegons = Mix_LoadWAV(sSoDeuSegons);
        if (*soDeuSegons==NULL)
        {
            cerr << "Error llegint <" << sSoDeuSegons << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
    sprintf(sSoFiPartida, "%s%s", sDirectori, "sound-endgame.ogg.wav");
    *soFiPartida = Mix_LoadWAV(sSoFiPartida);
    if (*soFiPartida==NULL)
    {
        sprintf(sSoFiPartida, "%s%s", sDirectoriDefecte, "sound-endgame.ogg.wav");
        *soFiPartida = Mix_LoadWAV(sSoFiPartida);
        if (*soFiPartida==NULL)
        {
            cerr << "Error llegint <" << sSoFiPartida << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
    sprintf(sSoNoMesLinies, "%s%s", sDirectori, "sound-nomorelines.ogg.wav");
    *soNoMesLinies = Mix_LoadWAV(sSoNoMesLinies);
    if (*soNoMesLinies==NULL)
    {
        sprintf(sSoNoMesLinies, "%s%s", sDirectoriDefecte, "sound-nomorelines.ogg.wav");
        *soNoMesLinies = Mix_LoadWAV(sSoNoMesLinies);
        if (*soNoMesLinies==NULL)
        {
            cerr << "Error llegint <" << sSoNoMesLinies << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
    sprintf(sSoPausa, "%s%s", sDirectori, "sound-pause.ogg.wav");
    *soPausa = Mix_LoadWAV(sSoPausa);
    if (*soPausa==NULL)
    {
        sprintf(sSoPausa, "%s%s", sDirectoriDefecte, "sound-pause.ogg.wav");
        *soPausa = Mix_LoadWAV(sSoPausa);
        if (*soPausa==NULL)
        {
            cerr << "Error llegint <" << sSoPausa << ">: " << Mix_GetError() << endl;
            exit(1);
        }
    }
}


void GestorTemes::CarregaFontTema(Uint8 tema, SDL_Surface** font, SDL_Surface* superficie)
{
    char sFont[128];

    sprintf(sFont, "data/themes/%s/font.png", mTemes[tema]);

    SDL_Surface* imgFont = IMG_Load(sFont);
    if (imgFont==NULL)
    {
        sprintf(sFont, "data/themes/default/font.png");
        imgFont = IMG_Load(sFont);
        if (imgFont==NULL)
        {
            cerr << "Error llegint <" << sFont << ">: " << SDL_GetError() << endl;
            exit(1);
        }
    }
    SDL_SetColorKey(imgFont, SDL_SRCCOLORKEY, SDL_MapRGB(superficie->format, 1, 0, 0));
    *font = SDL_DisplayFormat(imgFont);
    SDL_FreeSurface(imgFont);
}
